"use strict";
(self["webpackChunk_jupyter_server_documents"] = self["webpackChunk_jupyter_server_documents"] || []).push([["node_modules_jupyter_collaborative-drive_lib_index_js-_a2d01"],{

/***/ "./node_modules/@jupyter/collaborative-drive/lib/index.js":
/*!****************************************************************!*\
  !*** ./node_modules/@jupyter/collaborative-drive/lib/index.js ***!
  \****************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   ICollaborativeContentProvider: () => (/* reexport safe */ _tokens__WEBPACK_IMPORTED_MODULE_0__.ICollaborativeContentProvider),
/* harmony export */   IGlobalAwareness: () => (/* reexport safe */ _tokens__WEBPACK_IMPORTED_MODULE_0__.IGlobalAwareness)
/* harmony export */ });
/* harmony import */ var _tokens__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./tokens */ "./node_modules/@jupyter/collaborative-drive/lib/tokens.js");
/* -----------------------------------------------------------------------------
| Copyright (c) Jupyter Development Team.
| Distributed under the terms of the Modified BSD License.
|----------------------------------------------------------------------------*/
/**
 * @packageDocumentation
 * @module collaborative-drive
 */



/***/ }),

/***/ "./node_modules/@jupyter/collaborative-drive/lib/tokens.js":
/*!*****************************************************************!*\
  !*** ./node_modules/@jupyter/collaborative-drive/lib/tokens.js ***!
  \*****************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   ICollaborativeContentProvider: () => (/* binding */ ICollaborativeContentProvider),
/* harmony export */   IGlobalAwareness: () => (/* binding */ IGlobalAwareness)
/* harmony export */ });
/* harmony import */ var _lumino_coreutils__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @lumino/coreutils */ "webpack/sharing/consume/default/@lumino/coreutils");
/* harmony import */ var _lumino_coreutils__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(_lumino_coreutils__WEBPACK_IMPORTED_MODULE_0__);
// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.

/**
 * The collaborative drive.
 */
const ICollaborativeContentProvider = new _lumino_coreutils__WEBPACK_IMPORTED_MODULE_0__.Token('@jupyter/collaboration-extension:ICollaborativeContentProvider');
/**
 * The global awareness token.
 */
const IGlobalAwareness = new _lumino_coreutils__WEBPACK_IMPORTED_MODULE_0__.Token('@jupyter/collaboration:IGlobalAwareness');


/***/ })

}]);
//# sourceMappingURL=node_modules_jupyter_collaborative-drive_lib_index_js-_a2d01.46bb9de0528b8977dc6c.js.map